import tkinter as tk
from tkinter import messagebox, ttk
from math import floor
from sudokuai.SudokuBoard import SudokuBoard, SolveError

class SudokuGUI:
    def __init__(self, master):
        # super().__init__(master)
        self.master = master
        self.master.title("Sudoku Solver")

        self.entries = [[None for _ in range(9)] for _ in range(9)]
        self.stringVars = [[None for _ in range(9)] for _ in range(9)]
        self.separators = [[None, None] for _ in range(3)]
        self.started = False
        
        # initValues = [
        #     [0,8,1,0,3,0,9,0,0],
        #     [0,0,0,0,2,0,3,8,0],
        #     [3,2,6,9,7,0,1,4,5],
        #     [8,1,0,0,0,0,6,0,0],
        #     [6,0,0,8,0,7,0,9,0],
        #     [0,0,0,5,0,0,8,7,0],
        #     [0,6,0,7,5,4,0,0,0],
        #     [5,0,4,0,0,0,0,0,9],
        #     [2,7,8,1,0,0,0,6,4],
        # ]

        # initValues = [
        #     [0,0,0,0,4,0,0,2,6],
        #     [3,0,9,7,2,0,0,0,0],
        #     [0,5,0,0,0,0,4,0,0],
        #     [0,0,7,9,1,3,0,0,0],
        #     [6,0,3,0,0,0,0,0,0],
        #     [0,0,0,0,0,0,0,0,2],
        #     [1,0,0,0,0,0,0,8,0],
        #     [0,0,0,0,0,8,0,0,9],
        #     [0,9,0,0,0,0,7,0,4],
        # ]

        initValues = [
            [0,0,8,0,4,0,0,0,0],
            [0,0,2,0,0,1,4,0,3],
            [0,0,0,0,0,0,8,0,0],
            [0,0,0,9,0,0,5,6,0],
            [5,0,0,0,8,7,0,0,0],
            [0,0,0,2,0,0,0,9,0],
            [0,3,1,0,0,0,0,0,0],
            [0,9,0,0,0,2,7,0,0],
            [0,0,7,5,0,0,0,0,9],
        ]
        
        # Create the 9x9 grid of text entries
        for i in range(9):
            row = i + floor(i / 3)
            for j in range(9):
                col = j + floor(j / 3)
                var = tk.StringVar()
                entry = tk.Entry(master, width=2, font=('Arial', 24), justify='center', textvariable=var)
                entry.grid(row=row, column=col, padx=5, pady=5)
                self.entries[i][j] = entry
                self.stringVars[i][j] = var
                if initValues is not None and initValues[i][j] != 0:
                    var.set(str(initValues[i][j]))

        # Create separators
        for i in range(2):
            index = (i+1)*3+i
            self.separators[i][0] = ttk.Separator(master, orient='horizontal')
            self.separators[i][1] = ttk.Separator(master, orient='vertical')
            self.separators[i][0].grid(row=index,column=0,columnspan=11, padx=5, sticky='ew')
            self.separators[i][1].grid(row=0,column=index,rowspan=11, pady=5, sticky='ns')

        # Create buttons
        self.clear_button = tk.Button(master, text="Clear", command=self.clear_board)
        self.clear_button.grid(row=12, column=0, columnspan=2)

        self.start_button = tk.Button(master, text="Start", command=self.start_board)
        self.start_button.grid(row=12, column=2, columnspan=3)

        self.solve_button = tk.Button(master, text="Solve", command=self.solve_board)
        self.solve_button.grid(row=12, column=6, columnspan=3)

        self.check_button = tk.Button(master, text="Check", command=self.check_board)
        self.check_button.grid(row=12, column=10, columnspan=2)

    def clear_board(self):
        self.started = False
        for i in range(9):
            for j in range(9):
                if self.entries[i][j]['state'] == 'disabled':
                    self.entries[i][j]['state'] = 'normal'
                else:
                    self.stringVars[i][j].set('')

    def start_board(self):
        # Placeholder for check logic
        if not self.started:
            self.started = True
            for i in range(9):
                for j in range(9):
                    try:
                        strVal = self.stringVars[i][j].get()
                        if len(strVal) != 0:
                            try:
                                self.entries[i][j]['state'] = 'disabled'
                            except ValueError:
                                pass
                    except ValueError:
                        pass

    def getBoard(self):
        boardValues = [[0 for _ in range(9)] for _ in range(9)]
        for i in range(9):
            for j in range(9):
                try:
                    strVal = self.stringVars[i][j].get()
                    value = 0
                    if len(strVal) != 0:
                        try:
                            value = int(strVal)
                        except ValueError:
                            pass
                    boardValues[i][j] = value
                except ValueError:
                    pass
        return SudokuBoard(initialValues=boardValues)
    
    def setBoard(self,board):
        for i in range(9):
            for j in range(9):
                cellValues = board.board[i][j].numbers
                if len(cellValues) == 1:
                    self.stringVars[i][j].set(str(cellValues[0]))

    def solve_board(self):
        self.start_board()
        board = self.getBoard()
        try:
            for update in board.solve():
                pass
        except Exception as e:
            print(e)
        self.setBoard(board)

    def check_board(self):
        board = self.getBoard()
        for row in range(9):
            for col in range(9):
                self.entries[row][col].config(bg='white')
        cells = board.check()
        if cells is not None:
            row1 = int(cells[0].row)
            col1 = int(cells[0].column)

            row2 = int(cells[1].row)
            col2 = int(cells[1].column)

            self.entries[row1][col1].config(bg='red')
            self.entries[row2][col2].config(bg='red')

if __name__ == "__main__":
    root = tk.Tk()
    sudoku_gui = SudokuGUI(root)
    root.mainloop()